/*
 *  Lucidor - e-book reader
 *  Copyright © 2009-2014  Mikael Ylikoski
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var Lucifox = {
    onload: function() {
        window.removeEventListener("load", Lucifox.onload, false);
        document.getElementById("contentAreaContextMenu").
            addEventListener("popupshowing", Lucifox.show_context_menu, false);

	let prefs = Components.classes["@mozilla.org/preferences-service;1"].
	    getService(Components.interfaces.nsIPrefService).
	    getBranch("extensions.lucifox.");

	if (!prefs.prefHasUserValue("lastVersion")) {
	    //Lucifox.install_button();
	} else {
	    let ver = Lucifox.parse_version(prefs.getCharPref("lastVersion"));
	    /*
	    if (!Lucifox.is_updated(ver, [0, 8, 9])) {
		let cman = Components.classes["@lucidor.org/collectionManagerService;1"].getService().wrappedJSObject;
		cman.maybeAddCollection({ title: "Bokselskap.no",
					  type: "opds",
					  url: "http://www.bokselskap.no/wp-content/themes/bokselskap/tekster/opds/root.xml",
					  icon: null });
	    }
	    */
	    if (!Lucifox.is_updated(ver, [0, 9, 0])) {
		//Lucifox.install_button();
	    }
	}
	prefs.setCharPref("lastVersion", "0.9.10");
    },

    // Firefox 3.x
    get_content_document_3: function() {
	return gBrowser.selectedBrowser.contentDocument.wrappedJSObject;
    },

    // Firefox 4.0 and later
    get_content_document_4: function() {
	return gBrowser.selectedBrowser.contentDocument;
    },

    get_content_document: function() {
	if (gBrowser.selectedBrowser.contentDocument.wrappedJSObject
	    != undefined) {
	    Lucifox.get_content_document = Lucifox.get_content_document_3;
	    return Lucifox.get_content_document_3();
	}
	Lucifox.get_content_document = Lucifox.get_content_document_4;
	return Lucifox.get_content_document_4();
    },

    send_event: function(cmd) {
	if (gBrowser.selectedBrowser.contentDocument.documentURI.
	    substring(0, 37) != "chrome://lucidor/content/viewepub.xul")
	    return;

	let cd = Lucifox.get_content_document();
	let element = cd.createElement("LucidorDataElement");
	element.setAttribute("command", cmd);
	cd.documentElement.appendChild(element);

	let evt = cd.createEvent("Events");
	evt.initEvent("LucidorEvent", true, false);
	element.dispatchEvent(evt);
    },

    show_context_menu: function(event) {
	if (gBrowser.selectedBrowser.contentDocument.documentURI.
	    substring(0, 37) != "chrome://lucidor/content/viewepub.xul")
	    return;

	event.preventDefault();
	Lucifox.get_content_document().getElementById("contextmenu").
	    showPopup(null, event.screenX + 2, event.screenY + 2,
		      "context", null, null);
    },

    show_open: function() {
	let fp = Components.classes["@mozilla.org/filepicker;1"].
	    createInstance(Components.interfaces.nsIFilePicker);
	let sharedbundle = document.getElementById("lucifox-shared_strings");
	let text = sharedbundle.getString("open_book");
	fp.init(window, text, Components.interfaces.nsIFilePicker.modeOpen);
	text = sharedbundle.getString("books");
	fp.appendFilter(text, "*.epub ; *.EPUB");
	text = sharedbundle.getString("all_files");
	fp.appendFilter(text, "*");

	let callback = {
	    done: function(result) {
		if (result == fp.returnOK)
		    gBrowser.selectedTab = gBrowser.loadURI("chrome://lucidor/content/viewepub.xul?uri=" + encodeURIComponent(fp.fileURL.spec));
	    }
	};

	if (fp.open != undefined)
	    fp.open(callback);
	else
	    callback.done(fp.show());
    },

    show_opds: function() {
	gBrowser.selectedTab = gBrowser.loadURI("chrome://lucidor/content/viewopds.xul");
    },

    show_feed: function() {
	gBrowser.selectedTab = gBrowser.loadURI("chrome://lucidor/content/autopub.xul");
    },

    show_manual: function() {
	gBrowser.selectedTab = gBrowser.loadURI("chrome://lucidor/content/viewepub.xul?uri=" + encodeURIComponent("resource://lucidor/manual/lucifox-manual.epub"));
    },

    show_preferences: function() {
	window.openDialog("chrome://lucidor/content/preferences.xul",
			  "Preferences",	// LOCALIZE-ME?
			  "chrome,titlebar,toolbar,centerscreen,modal");
    },

    install_button: function() {
	let toolbarId = "addon-bar";
	let id = "lucifox-bar-button";
	let afterId = null;

	let appinfo = Components.classes["@mozilla.org/xre/app-info;1"].
	    getService(Components.interfaces.nsIXULAppInfo);
	if (appinfo.ID == "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}")	// SM
	    toolbarId = "nav-bar";

	if (!document.getElementById(id)) {
            let toolbar = document.getElementById(toolbarId);
	    if (!toolbar)
		return;

            let before = null;
            if (afterId) {
		let elem = document.getElementById(afterId);
		if (elem && elem.parentNode == toolbar)
                    before = elem.nextElementSibling;
            }

            toolbar.insertItem(id, before);
            toolbar.setAttribute("currentset", toolbar.currentSet);
            document.persist(toolbar.id, "currentset");

            if (toolbarId == "addon-bar")
		toolbar.collapsed = false;
	}
    },

    parse_version: function(ver) {
	let major = 0;
	let minor = 0;
	let extra = 0;
	try {
	    let parts = ver.split(".");
	    major = Number(parts[0]);
	    if (parts.length > 1)
		minor = Number(parts[1]);
	    if (parts.length > 2)
		extra = Number(parts[2]);
	} catch (e) {
	}
	return [major, minor, extra];
    },

    is_updated: function(prev, next) {
	if (prev[0] > next[0])			// major
	    return true;
	else if (prev[0] == next[0]) {		// major
	    if (prev[1] > next[1])		// minor
		return true;
	    else if (prev[1] == next[1]) {	// minor
		if (prev[2] >= next[2])		// extra
		    return true;
	    }
	}
	return false;
    }
};

window.addEventListener("load", Lucifox.onload, false);
