/*
 *  Lucidor - e-book reader
 *  Copyright © 2008-2014  Mikael Ylikoski
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Components.utils.import("chrome://lucidor/content/utils.jsm");
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function epubContentHandler(win, tempdir) {
    this.window = win;

    let bookman = Components.classes["@lucidor.org/bookmanagerService;1"].
	getService().wrappedJSObject;
    this.file = bookman.getTempfile(".epub");
    this.file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
			   parseInt("0600", 8));
    this.fos = Components.classes["@mozilla.org/network/file-output-stream;1"].
	createInstance(Components.interfaces.nsIFileOutputStream);
    this.fos.init(this.file, -1, -1, 0);
    this.out = Components.classes["@mozilla.org/binaryoutputstream;1"].
        createInstance(Components.interfaces.nsIBinaryOutputStream);
    this.out.setOutputStream(this.fos);
}

epubContentHandler.prototype = {
    window: null,
    file: null,
    fos: null,
    out: null,

    onDataAvailable: function(aRequest, aContext, aInputStream, aOffset, aCount) {
	let inp = Components.classes["@mozilla.org/binaryinputstream;1"].
            createInstance(Components.interfaces.nsIBinaryInputStream);
	inp.setInputStream(aInputStream);
	let data = inp.readBytes(aCount);
	this.out.writeBytes(data, aCount);
    },

    onStartRequest: function(aRequest, aContext) {
    },

    onStopRequest: function(aRequest, aContext, aStatusCode) {
	this.fos.close();
	this.out.close();

	let webnav = this.window.QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIWebNavigation);
	let urispec = "chrome://lucidor/content/viewepub.xul?uri=" +
	    encodeURIComponent(aRequest.name);

	if (aStatusCode != 0) {
	    // Show error page
	    return;
	}

	let furispec = Components.classes["@mozilla.org/network/protocol;1?name=file"].
	    getService(Components.interfaces.nsIFileProtocolHandler).
	    getURLSpecFromFile(this.file);
	let fn = null;
	try {
	    let chan = aRequest.QueryInterface(Components.interfaces.nsIChannel);
	    fn = chan.contentDispositionFilename;
	} catch (e) {
	}
	let bookman = Components.classes["@lucidor.org/bookmanagerService;1"].
	    getService().wrappedJSObject;
	bookman.addFile(this.window, furispec, fn);

	webnav.loadURI(urispec, 0, null, null, null);
    },

    QueryInterface: function(aIID) {
	if (aIID.equals(Components.interfaces.nsIStreamListener) ||
	    aIID.equals(Components.interfaces.nsIRequestObserver) ||
	    aIID.equals(Components.interfaces.nsISupports))
	    return this;
	throw Components.results.NS_NOINTERFACE;
    }
};

function epubXURIContentListener() {
    this.loadCookie = null;
    this.parentContentListener = null;
    this.wrappedJSObject = this;
    this.enable = true;
    try {
	this.prefs = Components.classes["@mozilla.org/preferences-service;1"].
            getService(Components.interfaces.nsIPrefService).
	    getBranch("extensions.lucifox.");
	this.enable = this.prefs.getBoolPref("enableOpener");
	this.prefs.addObserver("enableOpener", this, false);
    } catch (e) {
    }
}

epubXURIContentListener.prototype = {
    classDescription: "epubXURIContentListener",
    classID: Components.ID("{ec811fec-ec47-45ea-a395-c70fb1fc8f9d}"),
    contractID: "@lucidor.org/epubXURIContentListener;1",
    _xpcom_categories: [{ category: "app-startup" }],
    QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsISupports,
					   Components.interfaces.nsIURIContentListener,
					   Components.interfaces.nsIClassInfo,
					   Components.interfaces.nsISupportsWeakReference,
					   Components.interfaces.nsIObserver]),

    canHandleContent: function(aContentType, aIsContentPreferred) {
	if (this.enable && aContentType == "application/epub+zip")
	    return true;
	return false;
    },

    doContent: function(aContentType, aIsContentPreferred, aRequest, aContentHandler) {
	if (aContentType == "application/epub+zip") {
	    let window = aRequest.loadGroup.notificationCallbacks.
		getInterface(Components.interfaces.nsIDOMWindow);
	    if (aRequest.name.substring(0, 5) == "file:") {
		let urispec = "chrome://lucidor/content/viewepub.xul?uri=" +
		    encodeURIComponent(aRequest.name);
		let webnav = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor).getInterface(Components.interfaces.nsIWebNavigation);
		webnav.loadURI(urispec, 0, null, null, null);
		return true;
	    } else {
		aContentHandler.value = new epubContentHandler(window);
		return false;
	    }
	}

	// This should never happen
	//return false;
	throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    isPreferred: function(aContentType) {
	if (this.enable && aContentType == "application/epub+zip")
	    return true;
	return false;
    },

    onStartURIOpen: function(aURI) {
	return false;
    },

    getHelperForLanguage: function() {
    },

    getInterfaces: function() {
    },

    observe: function(subject, topic, data) {
	if (topic == "profile-after-change" || topic == "app-startup") {
	    let uriLoader = Components.classes["@mozilla.org/uriloader;1"].
		getService(Components.interfaces.nsIURILoader);
	    uriLoader.registerContentListener(contentListener);
	}

	if (topic == "nsPref:changed" && data == "enableOpener") {
	    this.enable = this.prefs.getBoolPref("enableOpener");

	    let uriLoader = Components.classes["@mozilla.org/uriloader;1"].
		getService(Components.interfaces.nsIURILoader);
	    uriLoader.unRegisterContentListener(contentListener);
	    uriLoader.registerContentListener(contentListener);
	}
    }
};

var contentListener = new epubXURIContentListener();

var components = [epubXURIContentListener];

if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule(components);
